using System;
using System.Windows.Forms;

public class StarLordsTwo {
		
	/// <summary>
	/// The StarLords Program    J M Bishop September 2002
	/// =====================
	/// simulates a battle, and has two levels of lords
	/// Illustrates inheritance, ref parameters and
	/// extensive use of Views
	/// </summary>

	Views.Form f;

	void SetUpForm() {
		try {
			f = new Views.Form(@"<form>
				<vertical>
				  <label text= 'The Battle of the Starlords' width=300/>
					<horizontal>
						<vertical>
						  <label name=A text='Brave Heart' height=20/>
							<picturebox image='Brave_Heart.jpg' height=60/>
						  <progressbar name=Astrength height=15/>
						  <progressbar name=Apoints height=15/>
						  <checkbox name='Aenhance' text=Enhance/>
						</vertical>
  					<vertical>
						  <label name=B text='Rob Roy' height=20/>
						  <picturebox image='Rob_Roy.jpg' height=60/>
							<progressbar name = Bstrength height=15/> 
						  <progressbar name = Bpoints height=15/> 
						  <checkbox name = 'Benhance' text =Enhance/>
					  </vertical>
          </horizontal>
			  <button name=Attack/>
				  <horizontal>
					  <label text='Winner'/>
					  <textbox name=Winner/>
				  </horizontal>
				  <button name=End/>
				</vertical>
			</form>" );
		} catch( Exception e ) {
			Console.WriteLine("Exception caught, message follows:{0}",
				e.Message);
		}
	}

	void DoBattle() {
		string c;
		for (;;) {
			c = f.GetControl();
			if (A.Points < 0 | B.Points < 0	| c=="End") break;
			ActionPerformed(c);
		}
		if (A.Points > B.Points) 
			f.PutText("Winner",A.Name);
		else
			f.PutText("Winner",B.Name);
		while (c != "End") {
			c = f.GetControl();
		}
		f.Dispose();
	}
	
	void ActionPerformed(string c) {
		switch (c) {
			case "Attack" : {
				if (AsTurn) 
					A.Attack(B);
				else 
					B.Attack(A);
				AsTurn=!AsTurn;
				Display(A,"A");
				Display(B,"B");
				break;
			}
			case "Aenhance" : {
				Enhance(ref A,"A");
				break;
			}
			case "Benhance" : {
				Enhance(ref B, "B");
				break;
			}
		}
	}

	Character A;
	Character B;

	void SetUpCharacters () {
		Label l = f["A"];
		A = new Character (l.Text, 80);
		l = f["B"];
		B = new Character (l.Text, 74);
		Display(A,"A");
		Display(B,"B");
	}

	void Enhance(ref Character c, string x) {
		int rememberPoints = c.Points;
		c = new SuperCharacter(c);
		c.Points = rememberPoints;
		ProgressBar p = f[x+"strength"];
		p.ForeColor = System.Drawing.Color.Red;
		Display(c,x);
	}

	bool AsTurn = true;

	void Go () {
		SetUpForm();
		SetUpCharacters();
		DoBattle();
	}


	void Display(Character lord, string x) {
		// inserts into the following sub-view
		//<progressbar name = xstrength text=''/>
		//<progressbar name = xpoints text=''/>
		Console.WriteLine(x+"  "+lord.Strength+"  "+lord.Points);
		f.PutValue(x+("strength"),lord.Strength);
		f.PutValue(x+("points"),lord.Points);
	}
	
	static void Main () {
		new StarLordsTwo().Go();
	}
}

	/// <summary>
	/// A class of StarLord characters with certain characteristics 
	/// and the  ability to to battle against other StarLords
	/// </summary>
	
public class Character {
	string name;
	int points;
	int strength;
	Random r = new Random ();
		
	public Character () {
		// required because of derived class
	}

	public Character (string n, int s) {
		name = n;
		strength = s;
		points = s;
	}

	public string Name {
		get {return name; }
	}

	public int Points {
		get {return points; }
		set {points = value;}
	}	 

	public int Strength {
		get {return strength; }
		set {strength = value;}
	}

	public virtual void Attack (Character opponent) {
		int damage = r.Next(strength/3+points/2);
		opponent.Points -= damage;
	}

	public override string ToString () {
		return name + " at " + points;
	}
}

	/// <summary>
	/// A derived class of StarLord characters with 
	/// enhanced attacking characteristics
	/// </summary>
	
public class SuperCharacter : Character {
	Random r = new Random ();
		
	public SuperCharacter (Character c) 
		// start with the same as before
		: base (c.Name, c.Strength) {
		// enhance strength
		Strength += r.Next(10);
	}

	// new attacking method
	// more strength (i.e. /2 not 3)
	// but more risky - there is a random +- factor
	public override void Attack (Character opponent) {
		// was int damage = r.Next(strength/3+points/2);
		int damage = r.Next(Strength/2+Points/2)-r.Next(1)*4; 
		opponent.Points -= damage;
	}
}
